import os.path
import shutil
import sys
from typing import Optional, List

import boto3

from johnsnowlabs import settings
from johnsnowlabs.auto_install.databricks.dbfs import dbfs_rm
from johnsnowlabs.auto_install.databricks.install_utils import (
    create_cluster,
    get_db_client_for_token,
    install_jsl_suite_to_cluster,
    install_list_of_pypi_ref_to_cluster,
)
from johnsnowlabs.auto_install.emr.install_utils import create_emr_cluster
from johnsnowlabs.auto_install.glue.install_utils import (
    create_glue_bucket,
    upload_pylibs_jars_to_glue_bucket,
)
from johnsnowlabs.auto_install.glue.utils import get_printable_glue_notebook_commands
from johnsnowlabs.auto_install.install_software import check_and_install_dependencies
from johnsnowlabs.auto_install.jsl_home import (
    get_install_suite_from_jsl_home,
    setup_jsl_home,
)
from johnsnowlabs.auto_install.offline_install import get_printable_dependency_urls
from johnsnowlabs.auto_install.softwares import Software
from johnsnowlabs.py_models.jsl_secrets import JslSecrets
from johnsnowlabs.utils.enums import JvmHardwareTarget, ProductName, PyInstallTypes


def install(
    # -- JSL-Auth Flows --
    # Browser Auth
    browser_login: bool = True,
    force_browser: bool = False,
    # JWT Token Auth
    access_token: Optional[str] = None,
    # JSON file Auth
    json_license_path: Optional[str] = None,
    # Manual License specification Auth
    med_license: Optional[str] = None,
    enterprise_nlp_secret: Optional[str] = None,
    ocr_secret: Optional[str] = None,
    ocr_license: Optional[str] = None,
    fin_license: Optional[str] = None,
    leg_license: Optional[str] = None,
    aws_access_key: Optional[str] = None,
    aws_key_id: Optional[str] = None,
    # -- Databricks auth flows & Install Target --
    databricks_cluster_id: Optional[str] = None,
    databricks_token: Optional[str] = None,
    databricks_host: Optional[str] = None,
    databricks_password: Optional[str] = None,
    databricks_email: Optional[str] = None,
    # -- Install Params --
    # Install Target
    python_exec_path: str = sys.executable,
    venv_creation_path: Optional[str] = None,
    offline_zip_dir: Optional[str] = None,
    # Download Params
    offline: bool = False,
    install_optional: bool = True,
    install_licensed: bool = True,
    slim_install: bool = False,  # Only Downloads jars
    product: Optional[str] = ProductName.jsl_full.value,
    include_dependencies: bool = True,
    nlp: bool = True,
    spark_nlp: bool = True,
    visual: bool = False,
    # License usage & Caching
    local_license_number: int = 0,
    remote_license_number: int = 0,
    store_in_jsl_home: bool = True,
    # Install File Types
    hardware_platform: str = JvmHardwareTarget.cpu.value,
    py_install_type: str = PyInstallTypes.wheel.value,
    only_refresh_credentials: bool = False,
    refresh_install: bool = False,
    # -- Databricks Cluster Creation Params --
    block_till_cluster_ready=True,
    num_workers=1,
    cluster_name=settings.db_cluster_name,
    node_type_id=settings.db_node_type_id,
    driver_node_type_id=settings.db_driver_node_type,
    spark_env_vars=None,
    autotermination_minutes=60,
    spark_version=settings.db_spark_version,
    spark_conf=None,
    auto_scale=None,
    aws_attributes=None,
    ssh_public_keys=None,
    custom_tags=None,
    cluster_log_conf=None,
    enable_elastic_disk=None,
    cluster_source=None,
    instance_pool_id=None,
    headers=None,
    clean_cluster=True,
    write_db_credentials=True,
    extra_pip_installs: Optional[List[str]] = None,
):
    if refresh_install and os.path.exists(settings.root_dir):
        print("🧹 Cleaning up old JSL Home in ", settings.root_dir)
        shutil.rmtree(settings.root_dir)
    if clean_cluster and databricks_host:
        dbfs_rm(
            get_db_client_for_token(databricks_host, databricks_token),
            settings.dbfs_home_dir,
            recursive=True,
        )

    # Input Validation
    py_install_type = PyInstallTypes.from_str(py_install_type)
    hardware_platform = JvmHardwareTarget.from_str(hardware_platform)
    product = Software.for_name(product)

    # Get Credentials from Auth Flow
    secrets: JslSecrets = JslSecrets.build_or_try_find_secrets(
        browser_login=browser_login,
        force_browser=force_browser,
        access_token=access_token,
        local_license_number=local_license_number,
        remote_license_number=remote_license_number,
        secrets_file=json_license_path,
        hc_license=med_license,
        hc_secret=enterprise_nlp_secret,
        ocr_secret=ocr_secret,
        ocr_license=ocr_license,
        aws_access_key=aws_access_key,
        aws_key_id=aws_key_id,
        return_empty_secrets_if_none_found=True,
        fin_license=fin_license,
        leg_license=leg_license,
        store_in_jsl_home=store_in_jsl_home,
    )
    if only_refresh_credentials:
        return

    if offline:
        # Offline Install
        get_printable_dependency_urls(
            secrets=secrets,
            jvm_install_type=hardware_platform,
            py_install_type=py_install_type,
        )
        if not offline_zip_dir:
            return

    if store_in_jsl_home and not offline:
        # Cache credentials, Wheels and Jars in ~/.johnsnowlabs
        setup_jsl_home(
            secrets=secrets,
            jvm_install_type=hardware_platform,
            py_install_type=py_install_type,
            refresh_install=refresh_install,
            visual=visual,
            nlp=nlp,
            spark_nlp=spark_nlp,
        )

    # Databricks Install
    if databricks_host and databricks_token and not offline:
        print(
            "nlp.install() for databricks will be deprecated next release, instead use nlp.install_to_databricks()"
        )
        suite = get_install_suite_from_jsl_home(
            jvm_hardware_target=hardware_platform,
            visual=visual,
            nlp=nlp,
            spark_nlp=spark_nlp,
        )
        if databricks_cluster_id:
            install_jsl_suite_to_cluster(
                db=get_db_client_for_token(databricks_host, databricks_token),
                install_suite=suite,
                cluster_id=databricks_cluster_id,
                medical_nlp=nlp,
                spark_nlp=spark_nlp,
                visual=visual,
            )
            if extra_pip_installs:
                install_list_of_pypi_ref_to_cluster(
                    db=get_db_client_for_token(databricks_host, databricks_token),
                    cluster_id=databricks_cluster_id,
                    pip_installs=extra_pip_installs,
                )

        else:
            return create_cluster(
                medical_nlp=nlp,
                spark_nlp=spark_nlp,
                visual=visual,
                databricks_host=databricks_host,
                databricks_token=databricks_token,
                install_suite=suite,
                block_till_cluster_ready=block_till_cluster_ready,
                num_workers=num_workers,
                cluster_name=cluster_name,
                node_type_id=node_type_id,
                driver_node_type_id=driver_node_type_id,
                spark_env_vars=spark_env_vars,
                autotermination_minutes=autotermination_minutes,
                spark_version=spark_version,
                spark_conf=spark_conf,
                auto_scale=auto_scale,
                aws_attributes=aws_attributes,
                ssh_public_keys=ssh_public_keys,
                custom_tags=custom_tags,
                cluster_log_conf=cluster_log_conf,
                enable_elastic_disk=enable_elastic_disk,
                cluster_source=cluster_source,
                instance_pool_id=instance_pool_id,
                headers=headers,
                write_db_credentials=write_db_credentials,
                extra_pip_installs=extra_pip_installs,
            )

    # Local Py-Install
    elif not slim_install:
        check_and_install_dependencies(
            product=product,
            secrets=secrets,
            install_optional=install_optional,
            install_licensed=install_licensed,
            python_exec_path=python_exec_path,
            py_setup_dir=venv_creation_path,
            offline_zip_dir=offline_zip_dir,
            include_dependencies=include_dependencies,
            visual=visual,
            spark_nlp=spark_nlp,
            enterpise_nlp=nlp,
        )


def install_to_databricks(
    # -- JSL-Auth Flows --
    # Browser Auth
    browser_login: bool = True,
    force_browser: bool = False,
    # JWT Token Auth
    access_token: Optional[str] = None,
    # JSON file Auth
    json_license_path: Optional[str] = None,
    # Manual License specification Auth
    med_license: Optional[str] = None,
    enterprise_nlp_secret: Optional[str] = None,
    ocr_secret: Optional[str] = None,
    ocr_license: Optional[str] = None,
    fin_license: Optional[str] = None,
    leg_license: Optional[str] = None,
    aws_access_key: Optional[str] = None,
    aws_key_id: Optional[str] = None,
    # -- Databricks auth flows & Install Target --
    databricks_cluster_id: Optional[str] = None,
    databricks_token: Optional[str] = None,
    databricks_host: Optional[str] = None,
    databricks_password: Optional[str] = None,
    databricks_email: Optional[str] = None,
    # -- Install Params --
    product: Optional[str] = ProductName.jsl_full.value,
    nlp: bool = True,
    spark_nlp: bool = True,
    visual: bool = False,
    # License usage & Caching
    local_license_number: int = 0,
    remote_license_number: int = 0,
    store_in_jsl_home: bool = True,
    # Install File Types
    hardware_platform: str = JvmHardwareTarget.cpu.value,
    py_install_type: str = PyInstallTypes.wheel.value,
    refresh_install: bool = False,
    # -- Databricks Cluster Creation Params --
    block_till_cluster_ready=True,
    num_workers=1,
    cluster_name=settings.db_cluster_name,
    node_type_id=settings.db_node_type_id,
    driver_node_type_id=settings.db_driver_node_type,
    spark_env_vars=None,
    autotermination_minutes=60,
    spark_version=settings.db_spark_version,
    spark_conf=None,
    auto_scale=None,
    aws_attributes=None,
    ssh_public_keys=None,
    custom_tags=None,
    cluster_log_conf=None,
    enable_elastic_disk=None,
    cluster_source=None,
    instance_pool_id=None,
    headers=None,
    clean_cluster=True,
    write_db_credentials=True,
    extra_pip_installs: Optional[List[str]] = None,
):
    #     :param extra_pip_installs: any extra libraries to install,
    #         e.g. ["pandas==1.2.3", "numpy", 'https://my.website.com/wheel.whl', '/path/to/my/local/wheel.whl']
    if refresh_install and os.path.exists(settings.root_dir):
        print("🧹 Cleaning up old JSL Home in ", settings.root_dir)
        shutil.rmtree(settings.root_dir)
    if clean_cluster and databricks_host:
        dbfs_rm(
            get_db_client_for_token(databricks_host, databricks_token),
            settings.dbfs_home_dir,
            recursive=True,
        )

    # Input Validation
    py_install_type = PyInstallTypes.from_str(py_install_type)
    hardware_platform = JvmHardwareTarget.from_str(hardware_platform)
    product = Software.for_name(product)

    # Get Credentials from Auth Flow
    secrets: JslSecrets = JslSecrets.build_or_try_find_secrets(
        browser_login=browser_login,
        force_browser=force_browser,
        access_token=access_token,
        local_license_number=local_license_number,
        remote_license_number=remote_license_number,
        secrets_file=json_license_path,
        hc_license=med_license,
        hc_secret=enterprise_nlp_secret,
        ocr_secret=ocr_secret,
        ocr_license=ocr_license,
        aws_access_key=aws_access_key,
        aws_key_id=aws_key_id,
        return_empty_secrets_if_none_found=True,
        fin_license=fin_license,
        leg_license=leg_license,
        store_in_jsl_home=store_in_jsl_home,
    )

    if store_in_jsl_home:
        # Cache credentials, Wheels and Jars in ~/.johnsnowlabs
        setup_jsl_home(
            secrets=secrets,
            jvm_install_type=hardware_platform,
            py_install_type=py_install_type,
            refresh_install=refresh_install,
            visual=visual,
            nlp=nlp,
            spark_nlp=spark_nlp,
        )

    # Databricks Install
    if not databricks_host and databricks_token:
        raise Exception()
    print(
        "nlp.install() for databricks will be deprecated next release, instead use nlp.install_to_databricks()"
    )
    suite = get_install_suite_from_jsl_home(
        jvm_hardware_target=hardware_platform,
        visual=visual,
        nlp=nlp,
        spark_nlp=spark_nlp,
    )
    if databricks_cluster_id:
        # Install to existing cluster
        install_jsl_suite_to_cluster(
            db=get_db_client_for_token(databricks_host, databricks_token),
            install_suite=suite,
            cluster_id=databricks_cluster_id,
            medical_nlp=nlp,
            spark_nlp=spark_nlp,
            visual=visual,
        )
        if extra_pip_installs:
            install_list_of_pypi_ref_to_cluster(
                db=get_db_client_for_token(databricks_host, databricks_token),
                cluster_id=databricks_cluster_id,
                pip_installs=extra_pip_installs,
            )

    else:
        # Create new cluster
        return create_cluster(
            medical_nlp=nlp,
            spark_nlp=spark_nlp,
            visual=visual,
            databricks_host=databricks_host,
            databricks_token=databricks_token,
            install_suite=suite,
            extra_pip_installs=extra_pip_installs,
            block_till_cluster_ready=block_till_cluster_ready,
            num_workers=num_workers,
            cluster_name=cluster_name,
            node_type_id=node_type_id,
            driver_node_type_id=driver_node_type_id,
            spark_env_vars=spark_env_vars,
            autotermination_minutes=autotermination_minutes,
            spark_version=spark_version,
            spark_conf=spark_conf,
            auto_scale=auto_scale,
            aws_attributes=aws_attributes,
            ssh_public_keys=ssh_public_keys,
            custom_tags=custom_tags,
            cluster_log_conf=cluster_log_conf,
            enable_elastic_disk=enable_elastic_disk,
            cluster_source=cluster_source,
            instance_pool_id=instance_pool_id,
            headers=headers,
            write_db_credentials=write_db_credentials,
        )


def install_to_emr(
    boto_session: Optional[boto3.Session] = None,
    # EMR specific configs
    bootstrap_bucket: Optional[str] = None,
    s3_logs_path: Optional[str] = None,
    service_role: Optional[str] = settings.emr_default_service_role,
    job_flow_role: Optional[str] = settings.emr_default_instance_profile,
    subnet_id: Optional[str] = None,
    ec2_key_name: Optional[str] = None,
    # Browser Auth
    browser_login: bool = True,
    force_browser: bool = False,
    # JWT Token Auth
    access_token: Optional[str] = None,
    # JSON file Auth
    json_license_path: Optional[str] = None,
    # Manual License specification Auth
    license: Optional[str] = None,
    aws_access_key: Optional[str] = None,
    aws_key_id: Optional[str] = None,
    local_license_number: int = 0,
    remote_license_number: int = 0,
    nlp: bool = True,
    spark_nlp: bool = True,
    visual: bool = False,
    hardware_platform: str = JvmHardwareTarget.cpu.value,
    auto_terminate_hours: Optional[int] = 1,
) -> str:
    """Install John Snow Labs NLP and selected products on an EMR cluster
    :param boto_session: Boto3 session. Refer to https://boto3.amazonaws.com/v1/documentation/api/latest/reference/core/session.html
    :param bootstrap_bucket: S3 bucket to store bootstrap scripts
    :param s3_logs_path: S3 path to store logs
    :param service_role: EMR service role
    :param job_flow_role: EMR job flow role / instance profilie
    :param subnet_id: EMR subnet id
    :param ec2_key_name: EC2 key name
    :param browser_login: Use browser login
    :param force_browser: Force browser login
    :param access_token: JWT access token
    :param json_license_path: Path to JSON license file
    :param license: License string
    :param local_license_number: Local license number
    :param remote_license_number: Remote license number
    :param nlp: Install NLP
    :param spark_nlp: Install Spark NLP
    :param visual: Install Visual
    :param hardware_platform: Hardware platform
    :param auto_terminate_hours : Idle hour to wait before terminating the cluster
    :return: EMR cluster id
    """
    secrets: JslSecrets = JslSecrets.build_or_try_find_secrets(
        browser_login=browser_login,
        force_browser=force_browser,
        access_token=access_token,
        local_license_number=local_license_number,
        remote_license_number=remote_license_number,
        secrets_file=json_license_path,
        hc_license=license,
        return_empty_secrets_if_none_found=True,
        fin_license=license,
        leg_license=license,
        aws_access_key=aws_access_key,
        aws_key_id=aws_key_id,
        only_return_secrets=True,
    )
    if not boto_session:
        boto_session = boto3.Session()

    cluster_id = create_emr_cluster(
        boto_session=boto_session,
        secrets=secrets,
        s3_logs_path=s3_logs_path,
        bootstrap_bucket=bootstrap_bucket,
        job_flow_role=job_flow_role,
        service_role=service_role,
        subnet_id=subnet_id,
        ec2_key_name=ec2_key_name,
        nlp=nlp,
        spark_nlp=spark_nlp,
        visual=visual,
        hardware_platform=hardware_platform,
        auto_terminate_hours=auto_terminate_hours,
    )
    return cluster_id


def install_to_glue(
    boto_session: Optional[boto3.Session] = None,
    glue_assets_bucket: Optional[str] = None,
    # Browser Auth
    browser_login: bool = True,
    force_browser: bool = False,
    # JWT Token Auth
    access_token: Optional[str] = None,
    # JSON file Auth
    json_license_path: Optional[str] = None,
    # Manual License specification Auth
    license: Optional[str] = None,
    aws_access_key: Optional[str] = None,
    aws_key_id: Optional[str] = None,
    local_license_number: int = 0,
    remote_license_number: int = 0,
    nlp: bool = True,
    spark_nlp: bool = True,
    visual: bool = False,
    hardware_platform: str = JvmHardwareTarget.cpu.value,
):
    """Install John Snow Labs NLP and selected products on a Glue job
    :param boto_session: Boto3 session. Refer to https://boto3.amazonaws.com/v1/documentation/api/latest/reference/core/session.html
    :param glue_assets_bucket: S3 bucket to store glue assets
    :param browser_login: Use browser login
    :param force_browser: Force browser login
    :param access_token: JWT access token
    :param json_license_path: Path to JSON license file
    :param license: License string
    :param local_license_number: Local license number
    :param remote_license_number: Remote license number
    :param nlp: Install NLP
    :param spark_nlp: Install Spark NLP
    :param visual: Install Visual
    :param hardware_platform: Hardware platform

    """
    if not boto_session:
        boto_session = boto3.Session()

    # Download jars and wheels so that we can upload them
    install_suite = get_install_suite_from_jsl_home(
        jvm_hardware_target=hardware_platform,
        visual=visual,
        nlp=nlp,
        spark_nlp=spark_nlp,
        browser_login=browser_login,
        force_browser=force_browser,
        access_token=access_token,
        local_license_number=local_license_number,
        remote_license_number=remote_license_number,
        secrets_file=json_license_path,
        hc_license=license,
        fin_license=license,
        leg_license=license,
        aws_access_key=aws_access_key,
        aws_key_id=aws_key_id,
    )
    # Make sure bucket exists
    bucket = create_glue_bucket(
        boto_session=boto_session,
        bucket=glue_assets_bucket,
    )
    # Upload jars and wheels to s3 bucket for glue
    jars_s3_location, packages_s3_location = upload_pylibs_jars_to_glue_bucket(
        boto_session=boto_session, install_suite=install_suite, bucket=bucket
    )

    # Show instructions on how to run a glue notebook
    print(
        get_printable_glue_notebook_commands(
            boto_session=boto_session,
            glue_assets_bucket=bucket,
            packages_s3_location=packages_s3_location,
            jars_s3_location=jars_s3_location,
            secrets=install_suite.secrets,
        )
    )



from typing import Tuple

import boto3
import botocore

from johnsnowlabs.utils.boto_utils import BotoException


def parse_s3_url(s3_url: str) -> Tuple[str, str]:
    """Parse s3 url and return bucket and key"""
    return s3_url.split("/")[2], "/".join(s3_url.split("/")[3:]).rstrip("/")


def create_bucket(boto_session: boto3.Session, bucket: str):
    """Create a bucket for EMR cluster logs
    :param boto_session: Botocore session
    :param bucket: Bucket name
    """
    try:
        s3_client = boto_session.client("s3")
        region = boto_session.region_name
        if region == "us-east-1":
            s3_client.create_bucket(
                Bucket=bucket,
            )
        else:
            s3_client.create_bucket(
                Bucket=bucket,
                CreateBucketConfiguration={"LocationConstraint": region},
            )
        return bucket
    except botocore.exceptions.ClientError as e:
        if e.response["Error"]["Code"] == "BucketAlreadyOwnedByYou":
            return bucket
        raise BotoException(
            code=e.response["Error"]["Code"], message=e.response["Error"]["Message"]
        )


def check_if_file_exists_in_s3(boto_session: boto3.Session, s3_url: str):
    """Check if file exists in s3 using s3 client
    :param boto_session : Botocore session
    :param s3_url: S3 url to check
    """
    try:
        s3_client = boto_session.client("s3")
        bucket, key = parse_s3_url(s3_url)
        s3_client.head_object(Bucket=bucket, Key=key)

        return True
    except botocore.exceptions.ClientError as e:
        return False
    except Exception:
        return False


def upload_file_to_s3(
    boto_session: boto3.Session, file_path: str, bucket: str, file_name: str
) -> str:
    """Upload a file to s3 bucket
    :botocore_session: Botocore session
    :param file_path: Path to file to upload
    :param bucket: Bucket to upload to
    :param file_name: File name to create
    :return s3_url: S3 url of uploaded file
    """
    try:
        s3_client = boto_session.client("s3")
        s3_client.upload_file(file_path, bucket, file_name)
        return f"s3://{bucket}/{file_name}"
    except botocore.exceptions.ClientError as e:
        raise BotoException(
            code=e.response["Error"]["Code"], message=e.response["Error"]["Message"]
        )


def upload_content(
    boto_session: boto3.Session, content: str, bucket: str, file_name: str
) -> str:
    """Upload content to s3 bucket
    :param boto_session: Botocore session
    :param content: Content to upload
    :param bucket: Bucket to upload to
    :param file_name: File name to create
    :return: s3_url: S3 url of uploaded file
    """

    try:
        s3_client = boto_session.client("s3")
        s3_client.put_object(Body=content, Bucket=bucket, Key=file_name)
        return f"s3://{bucket}/{file_name}"
    except botocore.exceptions.ClientError as e:
        raise BotoException(
            code=e.response["Error"]["Code"], message=e.response["Error"]["Message"]
        )



import inspect

import johnsnowlabs.auto_install.health_checks.endpoint_test as endp_test


def generate_endpoint_test(model, lic):
    # read source of endpoint_test.py and replace placeholders with actual values and return new source code
    return (
        inspect.getsource(endp_test)
        .replace("ENDPOINT LICENSE", lic)
        .replace("MODEL TO TEST", model)
    )



lic = """ENDPOINT LICENSE"""
model = """MODEL TO TEST"""
from johnsnowlabs import nlp

nlp.settings.enforce_versions = False


# def new_req():
#     """
#     :return: A list of default pip requirements for MLflow Models produced by this flavor.
#              Calls to :func:`save_model()` and :func:`log_model()` produce a pip environment
#              that, at minimum, contains these requirements.
#     """
#     import os
#     from johnsnowlabs import settings
#     from mlflow.utils.requirements_utils import _get_pinned_requirement
#
#     nlp.start()
#     _JOHNSNOWLABS_ENV_JSON_LICENSE_KEY = "JOHNSNOWLABS_LICENSE_JSON"
#     _JOHNSNOWLABS_ENV_HEALTHCARE_SECRET = "HEALTHCARE_SECRET"
#     _JOHNSNOWLABS_ENV_VISUAL_SECRET = "VISUAL_SECRET"
#     if (
#         _JOHNSNOWLABS_ENV_HEALTHCARE_SECRET not in os.environ
#         and _JOHNSNOWLABS_ENV_VISUAL_SECRET not in os.environ
#     ):
#         raise Exception(
#             f"You need to set the {_JOHNSNOWLABS_ENV_HEALTHCARE_SECRET} or {_JOHNSNOWLABS_ENV_VISUAL_SECRET} environment variable set."
#             f" Please contact John Snow Labs to get one"
#         )
#
#     _SPARK_NLP_JSL_WHEEL_URI = (
#         "https://pypi.johnsnowlabs.com/{secret}/spark-nlp-jsl/spark_nlp_jsl-"
#         + f"{settings.raw_version_medical}-py3-none-any.whl"
#     )
#
#     _SPARK_NLP_VISUAL_WHEEL_URI = (
#         "https://pypi.johnsnowlabs.com/{secret}/spark-ocr/"
#         f"spark_ocr-{settings.raw_version_ocr}-py3-none-any.whl"
#     )
#
#     deps = [
#         f"johnsnowlabs_for_databricks=={settings.raw_version_jsl_lib}",  # TODO UNDO THIS!!
#         _get_pinned_requirement("pyspark"),
#         # TODO remove pandas constraint when NLU supports it
#         # https://github.com/JohnSnowLabs/nlu/issues/176
#         "pandas<=1.5.3",
#         "nlu_by_ckl==5.0.2rc1",
#     ]
#
#     if _JOHNSNOWLABS_ENV_HEALTHCARE_SECRET in os.environ:
#         _SPARK_NLP_JSL_WHEEL_URI = _SPARK_NLP_JSL_WHEEL_URI.format(
#             secret=os.environ[_JOHNSNOWLABS_ENV_HEALTHCARE_SECRET]
#         )
#         deps.append(_SPARK_NLP_JSL_WHEEL_URI)
#
#     if _JOHNSNOWLABS_ENV_VISUAL_SECRET in os.environ:
#         _SPARK_NLP_VISUAL_WHEEL_URI = _SPARK_NLP_VISUAL_WHEEL_URI.format(
#             secret=os.environ[_JOHNSNOWLABS_ENV_VISUAL_SECRET]
#         )
#         deps.append(_SPARK_NLP_VISUAL_WHEEL_URI)
#     print("RETRNING DPES!!!!!!!!!!")
#     print(deps)
#     return deps


def print_query_df(query_df):
    print(query_df)
    for c in query_df.columns:
        print(query_df[c])


def find_writable_directory():
    import os

    # TODO this does not work
    # List of directories to check
    candidate_dirs = [os.getcwd()]

    for dir_path in candidate_dirs:
        if os.access(dir_path, os.W_OK):
            print(f"Found writable directory: {dir_path}")
            return dir_path
    print("Could not find a writable directory")
    return None


# mlflow.set_experiment(find_writable_directory())

import json


def write_string_to_json(input_string, filename="output.json"):
    with open(filename, "w") as json_file:
        json.dump({"data": input_string}, json_file)


def read_json_from_file(filename="output.json"):
    with open(filename, "r") as json_file:
        data = json.load(json_file)

    return data


def run_test():
    import mlflow
    import os
    from johnsnowlabs.auto_install.databricks.endpoints import (
        query_and_deploy_if_missing,
        delete_endpoint,
        nlu_name_to_endpoint,
    )

    # mlflow.johnsnowlabs.get_default_pip_requirements = new_req
    from johnsnowlabs import settings

    print("IN DB ????", settings.is_running_in_databricks())

    # write_string_to_json(lic, "lic.json")
    os.environ["HEALTHCARE_SECRET"] = json.loads(lic)["SECRET"]

    # mlflow.set_experiment(find_writable_directory())
    mlflow.set_experiment("/Users/christian@johnsnowlabs.com/my-experiment123")
    os.environ["JOHNSNOWLABS_LICENSE_JSON_FOR_CONTAINER"] = lic
    os.environ["JOHNSNOWLABS_LICENSE_JSON"] = lic

    print("USING LIC : ", lic)
    from johnsnowlabs.auto_install.jsl_home import get_install_suite_from_jsl_home

    get_install_suite_from_jsl_home(only_jars=True)
    # get_install_suite_from_jsl_home(only_jars=True, hc_secret=json.loads(lic)["SECRET"])
    # 1) one query to construct endpoint and run with default output level
    print_query_df(query_and_deploy_if_missing(model, "Hello World", True, True))

    # # 2) One query for every output level
    # for o in ["token", "sentence", "document"]:
    #     print_query_df(
    #         query_and_deploy_if_missing(model, "Hello World", output_level=o)
    #     )

    # 3) cleanup
    print(f"Deleting: {nlu_name_to_endpoint(model)}")
    delete_endpoint(
        nlu_name_to_endpoint(model),
        host=os.environ["DATABRICKS_HOST"],
        token=os.environ["DATABRICKS_TOKEN"],
    )


if __name__ == "__main__":
    dbutils.library.installPyPI("mlflow_by_johnsnowlabs")
    # dbutils.library.installPyPI("johnsnowlabs_by_ckl")
    run_test()



import json
from typing import Dict, Any


def dump_dataclass_to_json(
    data_class_instance, out_file_path, overwrite_if_exist: bool = True
):
    with open(out_file_path, "w", encoding="utf8") as json_file:
        json.dump(data_class_instance.__dict__, json_file, indent=4)


def json_path_as_dict(path) -> Dict[Any, Any]:
    with open(path, encoding="utf8") as f:
        return json.load(f)


def str_to_file(str_, path):
    with open(path, "w", encoding="utf8") as text_file:
        text_file.write(str_)
    return path


def file_to_str(path):
    with open(path, "r", encoding="utf8") as file:
        return file.read()


def path_tail(str_path):
    return str_path.split("/")[-1]


def path_head(str_path):
    return str_path.split("/")[0]



# Embedder Node compatible with haystack framework
import os
import sys
from pathlib import Path
from typing import List, Union

import numpy as np
from haystack.nodes.retriever import EmbeddingRetriever
from haystack.nodes.retriever._base_embedding_encoder import _BaseEmbeddingEncoder
from haystack.schema import Document


class _JohnsnowlabsEmbeddingEncoder(_BaseEmbeddingEncoder):
    def __init__(self, retriever: "EmbeddingRetriever"):
        # 1) Check imports
        try:
            from johnsnowlabs import nlp
            from nlu.pipe.pipeline import NLUPipeline
        except ImportError as exc:
            raise ImportError(
                "Could not import johnsnowlabs python package. "
                "Please install it with `pip install johnsnowlabs`."
            ) from exc

        # 2) Start a Spark Session
        try:
            os.environ["PYSPARK_PYTHON"] = sys.executable
            os.environ["PYSPARK_DRIVER_PYTHON"] = sys.executable
            nlp.start(hardware_target="gpu" if retriever.use_gpu else "cpu")
        except Exception as exc:
            raise Exception("Failure starting Spark Session") from exc
            # 3) Load the model
        try:
            self.embedding_model = nlp.load(retriever.embedding_model)
        except Exception as exc:
            raise Exception("Failure loading model") from exc

    def embed(self, texts: Union[List[str], str]) -> np.ndarray:
        return np.asarray(
            self.embedding_model.predict_embeds(texts),
            dtype=float,
        )

    def embed_queries(self, queries: List[str]) -> np.ndarray:
        return self.embed(queries)

    def embed_documents(self, docs: List[Document]) -> np.ndarray:
        return self.embed([d.content for d in docs])

    def train(
        **kwargs,
    ):
        raise NotImplementedError("Training not supported")

    def save(self, save_dir: Union[Path, str]):
        raise NotImplementedError("Saving not supported")


class JSLHaystackEmbeddings(EmbeddingRetriever):
    def __init__(self, **kwargs):
        inject()
        super().__init__(**kwargs)


def inject():
    # inject the emd encoder into haystack
    from haystack.nodes.retriever import _embedding_encoder

    _embedding_encoder._EMBEDDING_ENCODERS[
        "johnsnowlabs"
    ] = _JohnsnowlabsEmbeddingEncoder
    # inject the retriever into haystack


inject()



from typing import List, Optional

import boto3

from johnsnowlabs import settings
from johnsnowlabs.auto_install.softwares import Software
from johnsnowlabs.py_models.jsl_secrets import JslSecrets
from johnsnowlabs.utils.boto_utils import get_aws_used_creds


def get_printable_glue_notebook_commands(
    boto_session: boto3.Session,
    glue_assets_bucket: str,
    packages_s3_location: List[str],
    jars_s3_location: List[str],
    secrets: Optional[JslSecrets] = None,
):
    """Returns a printable string with the commands to be run in a Glue notebook
    :param boto_session: Boto3 session
    :param glue_assets_bucket: Glue assets bucket
    :param packages_s3_location: List of packages s3 locations
    :param jars_s3_location: List of jars s3 locations
    :param secrets: JSL secrets
    """
    region = boto_session.region_name
    register_listener = (
        (
            secrets.HC_LICENSE
            and "spark._jvm.com.johnsnowlabs.util.start.registerListenerAndStartRefresh()"
        )
        or (
            secrets.OCR_LICENSE
            and "spark._jvm.com.johnsnowlabs.util.OcrStart.registerListenerAndStartRefresh()"
        )
        or ""
    )

    aws_creds = get_aws_used_creds(boto_session)
    # Keys to be used for S3 access
    s3a_creds_conf = f"""--conf spark.hadoop.fs.s3a.access.key={aws_creds.access_key} 
--conf spark.hadoop.fs.s3a.secret.key={aws_creds.secret_key}"""
    if aws_creds.token:
        s3a_creds_conf += f""" 
--conf spark.hadoop.fs.s3a.session.token={aws_creds.token}"""

    if secrets and any([secrets.HC_LICENSE, secrets.OCR_LICENSE]):
        # Also append license and jsl keys
        jsl_secrets_conf = f""" 
--conf jsl.settings.license={secrets.HC_LICENSE or secrets.OCR_LICENSE} 
--conf spark.jsl.settings.pretrained.credentials.access_key_id={secrets.AWS_ACCESS_KEY_ID} 
--conf spark.jsl.settings.pretrained.credentials.secret_access_key={secrets.AWS_SECRET_ACCESS_KEY}"""

        s3a_creds_conf = s3a_creds_conf + jsl_secrets_conf

    return f"""

Add the following lines in the beginning of your Glue notebook job:

1.
%additional_python_modules {Software.jsl_lib.pypi_name}=={settings.raw_version_jsl_lib},{
   ",".join([path for path in packages_s3_location])
}

%extra_jars {",".join([path for path in jars_s3_location])}

%%configure 
{{
    "--conf": \"\"\"spark.jsl.settings.pretrained.cache_folder=s3://{glue_assets_bucket}/cache_pretrained/  
--conf spark.jars.packages=org.apache.hadoop:hadoop-aws:3.2.1,com.amazonaws:aws-java-sdk:1.11.828 
{s3a_creds_conf} 
--conf spark.hadoop.fs.s3a.aws.credentials.provider=org.apache.hadoop.fs.s3a.TemporaryAWSCredentialsProvider 
--conf spark.hadoop.fs.s3a.impl=org.apache.hadoop.fs.s3a.S3AFileSystem 
--conf spark.hadoop.fs.s3a.path.style.access=true 
--conf spark.jsl.settings.aws.region={region}\"\"\"
}}

2. 
%glue_version 4.0

import sys
from awsglue.transforms import *
from awsglue.utils import getResolvedOptions
from pyspark.context import SparkContext
from awsglue.context import GlueContext
from awsglue.job import Job

sc = SparkContext.getOrCreate()
glueContext = GlueContext(sc)
spark = glueContext.spark_session
{register_listener}
job = Job(glueContext)
"""



from johnsnowlabs import lab, settings, viz
from johnsnowlabs.abstract_base.lib_resolver import try_import_lib
from johnsnowlabs.auto_install.databricks.endpoints import query_and_deploy_if_missing
from johnsnowlabs.auto_install.databricks.work_utils import run_in_databricks
from johnsnowlabs.auto_install.emr.work_utils import run_in_emr

from .auto_install.health_checks.report import (
    check_health,
    list_local_licenses,
    list_remote_licenses,
)
from .auto_install.install_flow import (
    install,
    install_to_emr,
    install_to_glue,
    install_to_databricks,
)
from .utils.sparksession_utils import start

if try_import_lib("sparknlp"):
    import sparknlp
    from sparknlp import annotation
    from sparknlp.annotator import *
    from sparknlp.base import *
    from sparknlp.functions import *
    from sparknlp.pretrained import PretrainedPipeline, ResourceDownloader
    from sparknlp.training import *


if try_import_lib("pyspark"):
    import pyspark.ml.param.shared as _shared_pyspark_ml_param
    import pyspark.sql as SQL
    import pyspark.sql.functions as F
    import pyspark.sql.types as T
    from pyspark import ml as ML
    from pyspark.sql import DataFrame

    ML.param.shared = _shared_pyspark_ml_param

    from pyspark.ml import Pipeline, PipelineModel
    from pyspark.sql import SparkSession

if try_import_lib("warnings"):
    import warnings

    warnings.filterwarnings("ignore")

if try_import_lib("nlu"):
    import nlu as nlu
    from nlu import autocomplete_pipeline, load, to_nlu_pipe, to_pretty_df



from johnsnowlabs import try_import_lib

if try_import_lib("haystack"):
    from johnsnowlabs.frameworks.embedding_retrival.haystack_node import (
        JSLHaystackEmbeddings,
    )

if try_import_lib("langchain"):
    from johnsnowlabs.frameworks.embedding_retrival.langchain_node import (
        JSLLangChainEmbeddings,
    )


if not try_import_lib("langchain") and not try_import_lib("haystack"):
    print(
        "Install either langchain or haystack and restart your python kernel to use the llm module!"
    )



import inspect
import os.path
import time
from pathlib import Path
from types import ModuleType
from typing import Callable, Optional, List, Any

from johnsnowlabs.auto_install.databricks.dbfs import *
from johnsnowlabs.auto_install.databricks.install_utils import create_cluster
from johnsnowlabs.utils.file_utils import path_tail, str_to_file


class LifeCycleState:
    PENDING = "PENDING"
    RUNNING = "RUNNING"
    TERMINATING = "TERMINATING"
    TERMINATED = "TERMINATED"
    SKIPPED = "SKIPPED"
    INTERNAL_ERROR = "INTERNAL_ERROR"
    BLOCKED = "BLOCKED"
    WAITING_FOR_RETRY = "WAITING_FOR_RETRY"

    end_states = [
        TERMINATING,
        TERMINATED,
        SKIPPED,
        INTERNAL_ERROR,
        BLOCKED,
    ]

    wait_states = [
        PENDING,
        RUNNING,
    ]


class ResultState:
    SUCCESS = "SUCCESS"
    FAILED = "FAILED"
    TIMEDOUT = "TIMEDOUT"
    CANCELED = "CANCELED"
    MAXIMUM_CONCURRENT_RUNS_REACHED = "MAXIMUM_CONCURRENT_RUNS_REACHED"
    EXCLUDED = "EXCLUDED"
    SUCCESS_WITH_FAILURES = "SUCCESS_WITH_FAILURES"
    UPSTREAM_FAILED = "UPSTREAM_FAILED"
    UPSTREAM_CANCELED = "UPSTREAM_CANCELED"

    wait_states = []

    end_states = [
        SUCCESS,
        FAILED,
        FAILED,
        TIMEDOUT,
        CANCELED,
        MAXIMUM_CONCURRENT_RUNS_REACHED,
        EXCLUDED,
        SUCCESS_WITH_FAILURES,
        UPSTREAM_FAILED,
        UPSTREAM_CANCELED,
    ]


def create_job_in_databricks(
    db: DatabricksAPI,
    local_python_script: str = None,
    cluster_id=None,
    name=None,
    new_cluster=None,
    libraries=None,
    email_notifications=None,
    timeout_seconds=None,
    max_retries=None,
    min_retry_interval_millis=None,
    retry_on_timeout=None,
    schedule=None,
    notebook_task=None,
    spark_jar_task=None,
    # spark_python_task=None,
    spark_submit_task=None,
    max_concurrent_runs=None,
    tasks=None,
    headers=None,
    version=None,
):
    #     https://docs.databricks.com/dev-tools/api/latest/jobs.html#operation/JobsCreate
    dbfs_target_path = copy_from_local_to_hdfs(
        db=db,
        local_path=local_python_script,
        dbfs_path=get_db_path(local_python_script),
    )
    if not name:
        name = settings.db_job_name.format(
            job=get_db_path(local_python_script).split("/")[-1]
        )

    if not cluster_id:
        raise NotImplementedError("Not Cluster ID based install not yet implemented")

    db.jobs.create_job(
        name=name,
        spark_python_task=dbfs_target_path,
        notebook_task=notebook_task,
        spark_jar_task=spark_jar_task,
        spark_submit_task=spark_submit_task,
        libraries=libraries,
        existing_cluster_id=cluster_id,
        new_cluster=new_cluster,
        email_notifications=email_notifications,
        timeout_seconds=timeout_seconds,
        max_retries=max_retries,
        min_retry_interval_millis=min_retry_interval_millis,
        retry_on_timeout=retry_on_timeout,
        schedule=schedule,
        max_concurrent_runs=max_concurrent_runs,
        tasks=tasks,
        headers=headers,
        version=version,
    )


def run_local_py_script_as_task(
    db: DatabricksAPI,
    task_definition: Union[str, ModuleType, Callable],
    cluster_id: str = None,
    run_name: str = None,
    parameters: List[Any] = None,
    dst_path=None,
):
    """
    # https://docs.databricks.com/dev-tools/api/latest/examples.html#jobs-api-examples
    A job consists of 1 or more tasks
    :param db:
    :param task_definition:
    :param cluster_id:
    :param run_name:
    :return:
    """
    if not run_name:
        run_name = settings.db_run_name

    if not cluster_id:
        cluster_id = create_cluster(db)

    if (
        isinstance(task_definition, str)
        and os.path.exists(task_definition)
        and task_definition.endswith(".ipynb")
    ):
        # Notebook task, WIP
        # raise Exception("Notebook task not supported yet")
        if not dst_path:
            raise Exception("dst_path must be provided for notebook tasks")
        file_name = os.path.split(task_definition)[-1]
        run_id = submit_notebook_to_databricks(
            db, task_definition, cluster_id, dst_path, parameters
        )

    else:
        task_definition = executable_as_script(task_definition)

        copy_from_local_to_hdfs(
            db=db,
            local_path=task_definition,
            dbfs_path=get_db_path(task_definition) if not dst_path else dst_path,
        )
        py_task = dict(
            python_file=get_db_path(task_definition),
        )
        if parameters:
            py_task["parameters"] = parameters

        run_id = db.jobs.submit_run(
            existing_cluster_id=cluster_id,
            spark_python_task=py_task,
            run_name=run_name,
        )["run_id"]
    print(f"Stated task with run_id={run_id}")
    return run_id


def executable_as_script(py_executable: Union[str, ModuleType, Callable]):
    if (
        isinstance(py_executable, str)
        and os.path.exists(py_executable)
        and py_executable.endswith(".py")
    ):
        print(f"Detected Python Script for Databricks Submission")
        # Py file, we can just run this
        return py_executable
    if (
        isinstance(py_executable, str)
        and os.path.exists(py_executable)
        and py_executable.endswith(".ipynb")
    ):
        print(f"Detected Python Notebook for Databricks Submission")
        # Py file, we can just run this
        return py_executable
    if isinstance(py_executable, (str, ModuleType, Callable)):
        print(f"Module/Callable for Databricks Submission")
        # Convert Module/Callable into a script
        return executable_to_str(py_executable)
    raise TypeError(f"Invalid Executable Python  Task {py_executable}")


def executable_to_str(executable_to_convert: Union[str, ModuleType, Callable]):
    # write a python code-string/module/function into a temp file and return resulting python file
    import johnsnowlabs.utils.testing.test_settings

    Path(johnsnowlabs.utils.testing.test_settings.tmp_notebook_dir).mkdir(
        parents=True, exist_ok=True
    )
    from random import randrange

    if isinstance(executable_to_convert, str) and not os.path.exists(
        executable_to_convert
    ):
        # Executable script
        file_name = f"{randrange(1333777)}tmp.py"

    else:
        # Module/Callable
        try:
            file_name = path_tail(inspect.getfile(executable_to_convert))
        except:
            # Within a Python shell, we cannot getFile(), so we have this fallback name
            file_name = f"{randrange(1333777)}tmp.py"

    out_path = (
        f"{johnsnowlabs.utils.testing.test_settings.tmp_notebook_dir}/{file_name}"
    )

    if isinstance(executable_to_convert, str):
        print(f"Detected Python Code String")
        return str_to_file(executable_to_convert, out_path)

    if isinstance(executable_to_convert, Callable):
        print(f"Detected Python Function for Databricks Submission")
        return str_to_file(
            inspect.getsource(executable_to_convert)
            + f"\n{executable_to_convert.__name__}()",
            out_path,
        )

    if isinstance(executable_to_convert, ModuleType):
        print(f"Detected Python Module for Databricks Submission")
        return str_to_file(inspect.getsource(executable_to_convert), out_path)


def encode_nb(nb_path):
    import base64

    with open(nb_path, "rb") as f:
        encoded_nb = base64.b64encode(f.read()).decode("utf-8")
    return encoded_nb


def import_notebook_to_workspace(
    db: DatabricksAPI,
    dst_path: str,
    local_nb_path: str,
):
    db.workspace.import_workspace(
        path=dst_path,
        language="PYTHON",
        format="JUPYTER",
        content=encode_nb(local_nb_path),
        overwrite=True,
    )


def checkon_db_task(
    db: DatabricksAPI,
    run_id: str = None,
):
    """
    # https://docs.databricks.com/dev-tools/api/latest/examples.html#jobs-api-examples
    A job consists of 1 or more tasks
    :param db:
    :param task_definition:
    :param cluster_id:
    :param run_name:
    :return:
    """
    # https://docs.databricks.com/dev-tools/api/2.0/jobs.html#runstate
    return db.jobs.get_run(run_id=run_id)


def run_in_databricks(
    py_script_path: Union[str, ModuleType, Callable],
    databricks_cluster_id: Optional[str] = None,
    databricks_token: Optional[str] = None,
    databricks_host: Optional[str] = None,
    run_name: str = None,
    block_till_complete=True,
    dst_path: str = None,
    parameters: Any = None,
):
    """

    :param py_script_path: Either
        - Path to local python script i.e "path/to/my_script.py"
        - Path to local python notebook i.e "path/to/my_notebook.ipynb"
        - string with python code i.e. "1+1"
        - python module i.e. import my_module
        - python function, i.e. def my_func(): return 1+1

    :param databricks_cluster_id: cluster ID to run your task on. If none provided, a cluster will be created first.
    :param databricks_token: databricks access token
    :param databricks_host: databricks host
    :param run_name: name of the run to generate, if None, a random name will be generated
    :param block_till_complete: if True, this function will block until the task is complete
    :param dst_path: path to store the python script/notebook. in databricks, mandatory for notebooks.
        I.e. /Users/<your@databricks.email.com>/test.ipynb
    :param parameters: parameters to pass to the python script/notebook formatted accordingly to https://docs.databricks.com/en/workflows/jobs/create-run-jobs.html#pass-parameters-to-a-databricks-job-task
    :return: job_id
    """
    from johnsnowlabs.auto_install.databricks.install_utils import (
        get_db_client_for_token,
    )

    db_client = get_db_client_for_token(databricks_host, databricks_token)
    job_id = run_local_py_script_as_task(
        db_client,
        py_script_path,
        cluster_id=databricks_cluster_id,
        run_name=run_name,
        dst_path=dst_path,
        parameters=parameters,
    )

    if not block_till_complete:
        return job_id
    job_done = False
    link_logged = False
    while not job_done:
        job_status = checkon_db_task(db_client, job_id)
        if not link_logged and "run_page_url" in job_status:
            print(f"You can monitor the job at {job_status['run_page_url']}")
            link_logged = True

        if "result_state" in job_status["state"]:
            print(f"Job has a result! its {job_status['state']}")
            return job_status
        elif "life_cycle_state" in job_status["state"]:
            print(
                f"Waiting 30 seconds, job {job_id}  is still running, its {job_status['state']}"
            )
        time.sleep(30)

    return job_id


def submit_notebook_to_databricks(
    db: DatabricksAPI, local_nb_path, cluster_id, remote_path, parameters=None
):
    # Instantiate the DatabricksAPI client
    # Read the local notebook content
    with open(local_nb_path, "rb") as file:
        content = file.read()

    # Base64 encode the content
    encoded_content = base64.b64encode(content).decode("utf-8")

    # Define the path on Databricks workspace where the notebook will be uploaded

    # Import the notebook to Databricks workspace
    db.workspace.import_workspace(
        path=remote_path,
        format="JUPYTER",
        language="PYTHON",
        content=encoded_content,
        overwrite=True,
    )

    # Define the notebook task
    notebook_task = {"notebook_path": remote_path}

    if parameters:
        notebook_task["base_parameters"] = parameters

    # Submit the job to run the notebook
    response = db.jobs.submit_run(
        run_name="Notebook_Run",
        existing_cluster_id=cluster_id,
        notebook_task=notebook_task,
    )

    return response["run_id"]



from codecs import open
from os import path

from setuptools import find_packages, setup

import johnsnowlabs.settings

here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

REQUIRED_PKGS = [
    f"pyspark=={johnsnowlabs.settings.raw_version_pyspark}",
    f"spark-nlp=={johnsnowlabs.settings.raw_version_nlp}",
    f"nlu=={johnsnowlabs.settings.raw_version_nlu}",
    f"spark-nlp-display=={johnsnowlabs.settings.raw_version_nlp_display}",
    "numpy",
    "dataclasses",
    "requests",
    "databricks-api",
    f"pydantic=={johnsnowlabs.settings.raw_version_pydantic}",
    "colorama",
    "boto3",
]

setup(
    version=johnsnowlabs.settings.raw_version_jsl_lib,
    name="johnsnowlabs",
    description="The John Snow Labs Library gives you access to all of John Snow Labs Enterprise And Open Source products in an easy and simple manner. Access 10000+ state-of-the-art NLP and OCR models for "
    "Finance, Legal and Medical domains. Easily scalable to Spark Cluster ",
    long_description=long_description,
    install_requires=REQUIRED_PKGS,
    long_description_content_type="text/markdown",
    url="https://www.johnsnowlabs.com/",
    author="John Snow Labs",
    author_email="christian@johnsnowlabs.com",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: Apache Software License",
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
    ],
    keywords="Spark NLP OCR Finance Legal Medical John Snow Labs  ",
    packages=find_packages(exclude=["test*", "tmp*"]),  # exclude=['test']
    include_package_data=True,
)



import os
import sys
from typing import Any, List

from langchain.embeddings.base import Embeddings

# from langchain.pydantic_v1 import BaseModel, Extra
from pydantic import BaseModel, Extra


class JSLLangChainEmbeddings(BaseModel, Embeddings):
    """JohnSnowLabs embedding models

    To use, you should have the ``johnsnowlabs`` python package installed.
    Example:        .. code-block:: python
            from langchain.embeddings.johnsnowlabs import JohnSnowLabsEmbeddings
            document = "foo bar"
            embedding = JohnSnowLabsEmbeddings('embed_sentence.bert')
            output = embedding.embed_query(document)
    """

    async def aembed_documents(self, texts: List[str]) -> List[List[float]]:
        raise NotImplementedError("JohnSnowLabsEmbeddings does not support async yet")

    async def aembed_query(self, text: str) -> List[float]:
        raise NotImplementedError("JohnSnowLabsEmbeddings does not support async yet")

    model: Any

    def __init__(
        self, model="embed_sentence.bert", hardware_target="cpu", **kwargs: Any
    ):
        """Initialize the johnsnowlabs model."""
        super().__init__(**kwargs)
        # 1) Check imports
        try:
            from johnsnowlabs import nlp
            from nlu.pipe.pipeline import NLUPipeline
        except ImportError as exc:
            raise ImportError(
                "Could not import johnsnowlabs python package. "
                "Please install it with `pip install johnsnowlabs`."
            ) from exc

            # 2) Start a Spark Session
        try:
            os.environ["PYSPARK_PYTHON"] = sys.executable
            os.environ["PYSPARK_DRIVER_PYTHON"] = sys.executable
            nlp.start(hardware_target=hardware_target)
        except Exception as exc:
            raise Exception("Failure starting Spark Session") from exc
            # 3) Load the model
        try:
            if isinstance(model, str):
                self.model = nlp.load(model)
            elif isinstance(model, NLUPipeline):
                self.model = model
            else:
                self.model = nlp.to_nlu_pipe(model)
        except Exception as exc:
            raise Exception("Failure loading model") from exc

    class Config:
        """Configuration for this pydantic object."""

        extra = Extra.forbid

    def embed_documents(self, texts: List[str]) -> List[List[float]]:
        """Compute doc embeddings using a JohnSnowLabs transformer model.

        Args:            texts: The list of texts to embed.
        Returns:            List of embeddings, one for each text."""

        return self.model.predict_embeds(texts)

    def embed_query(self, text: str) -> List[float]:
        """Compute query embeddings using a JohnSnowLabs transformer model.
        Args:            text: The text to embed.
        Returns:            Embeddings for the text."""
        return self.model.predict_embeds(text)[0]
